/*
 * Copyright (C) 2012  Arnaud Bonatti  <arnaud.bonatti@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
 * USA.
 */

const Gio			= imports.gi.Gio
const GLib			= imports.gi.GLib
const Main			= imports.ui.main
const Meta			= imports.gi.Meta
const Search		= imports.ui.search
const Shell			= imports.gi.Shell
const Util			= imports.misc.util

const MainFile		= imports.misc.extensionUtils.getCurrentExtension().imports.extension	// local file
const dconfSchema	= 'org.gnome.shell.extensions.terminal-provider'
const gettextDomain = 'terminal-provider'
const _				= imports.gettext.domain(gettextDomain).gettext

// strings
const LAUNCH_TEXT	= "Launch";				let LAUNCH_TEXT_lc,		LAUNCH_TEXT_lc_i18n
const OPEN_TEXT		= "Open in terminal";	let OPEN_TEXT_lc, 		OPEN_TEXT_lc_i18n
const RESTART_TEXT	= "Restart Shell";		let RESTART_TEXT_lc,	RESTART_TEXT_lc_i18n
const LG_TEXT		= "Looking Glass";		let LG_TEXT_lc, 		LG_TEXT_lc_i18n

/* Remove <Alt>F2, one time */

const SHELLENTRYKEY = 'org.gnome.desktop.wm.keybindings'

let lockFilePath = null
let connection = null
let seSettings = null

function rEnableShellEntry(createLockFile) {
	Meta.keybindings_set_custom_handler('panel-run-dialog', function() { Main.getRunDialog().open() })
	seSettings.disconnect(connection)
	
	if (createLockFile) {
		let file = Gio.file_new_for_path(lockFilePath)
		let out = file.replace(null, false, Gio.FileCreateFlags.NONE, null)
		Shell.write_string_to_stream (out, "true")
	} else
		seSettings.set_strv('panel-run-dialog', MainFile.getSettings(dconfSchema).get_strv('shell-entry-keybinding'))
	
	connection = null
 	seSettings = null
}

function disableShellEntry() {
	seSettings = new Gio.Settings({ 'schema': SHELLENTRYKEY })
	MainFile.getSettings(dconfSchema).set_strv('shell-entry-keybinding', seSettings.get_strv('panel-run-dialog'))
	seSettings.set_strv('panel-run-dialog', [])		// for the control-center not showing the old keybinding

	// not possible to play directly with built-in keybindings (22-08-2012, -> 3.6), so lets play a little
	Meta.keybindings_set_custom_handler('panel-run-dialog', function() {})
	connection = seSettings.connect('changed', function() { if (seSettings.get_strv('panel-run-dialog')!="") rEnableShellEntry(true) })
}

/* Terminal Provider */
const OVERVIEWTITLE = "TERMINAL"				// Title in the Overview
const GNOMETERMICON = "gnome-terminal.desktop"	// Icon used
const TERMS = ["gnome-terminal", "xfce4-terminal", "xterm"]

function TerminalExec() { this._init() }

TerminalExec.prototype = {
	__proto__: Search.SearchProvider.prototype,

	_init: function() {
		Search.SearchProvider.prototype._init.call(this, OVERVIEWTITLE)
	},

	getResultMetas: function(ids, callback) {
		let appSys = Shell.AppSystem.get_default()
		let app = appSys.lookup_app(GNOMETERMICON)
		
		let cb = new Array( { 'id': "open",		'name': _(OPEN_TEXT),		'createIcon': function(size) { return app.create_icon_texture(size) }},
							{ 'id': "launch",	'name': _(LAUNCH_TEXT),		'createIcon': function(size) { return app.create_icon_texture(size) }})
		
		if (ids.length > 2) {
			if (ids[0] == "lg")
				cb.unshift(	{ 'id': "lg",		'name': _(LG_TEXT),			'createIcon': function(size) { return app.create_icon_texture(size) }})
			else // ids[2] == "r"
				cb.push(	{ 'id': "r",		'name': _(RESTART_TEXT),	'createIcon': function(size) { return app.create_icon_texture(size) }})
		}
		callback(cb)
	},

	getInitialResultSet: function(terms) {							// same as getSubsearchResultSet(...)
		this.terms = terms
		
		let term = terms.join(' ').toLowerCase()
		if		(term == LG_TEXT_lc_i18n.substr(0, term.length))		this.searchSystem.pushResults(this, ["lg", "open", "launch"])
		else if	(term == RESTART_TEXT_lc_i18n.substr(0, term.length))	this.searchSystem.pushResults(this, ["open", "launch", "r"])
		else if	(term == LG_TEXT_lc.substr(0, term.length))				this.searchSystem.pushResults(this, ["lg", "open", "launch"])
		else if	(term == RESTART_TEXT_lc.substr(0, term.length))		this.searchSystem.pushResults(this, ["open", "launch", "r"])
		else															this.searchSystem.pushResults(this, ["open", "launch"])
	},
	getSubsearchResultSet: function(previousResults, terms) {		// same as getInitialResultSet(terms)
		this.terms = terms
		
		let term = terms.join(' ').toLowerCase()
		if		(term == LG_TEXT_lc_i18n.substr(0, term.length))		this.searchSystem.pushResults(this, ["lg", "open", "launch"])
		else if	(term == RESTART_TEXT_lc_i18n.substr(0, term.length))	this.searchSystem.pushResults(this, ["open", "launch", "r"])
		else if	(term == LG_TEXT_lc.substr(0, term.length))				this.searchSystem.pushResults(this, ["lg", "open", "launch"])
		else if	(term == RESTART_TEXT_lc.substr(0, term.length))		this.searchSystem.pushResults(this, ["open", "launch", "r"])
		else															this.searchSystem.pushResults(this, ["open", "launch"])
	},
	activateResult: function(id) {
		if		(id == "r")  global.reexec_self()
		else if (id == "lg") Main.createLookingGlass().toggle()
		else {
			let command = this.terms.join(' ')
			if (id == "open") {
				let i = 0, error
				do {
					error = null
					try {
						if (TERMS[i+1])	Util.trySpawn([TERMS[i], "-e", command])
						else /*xterm*/	Util.spawn([TERMS[i], "-e", command])
					} catch(fail) {
						error = fail
						global.log(TERMS[i] + " not found (extension306.js:136). Trying " + TERMS[i+1] + ".")
						i += 1
					}
				} while (error)
			} else /* if (id == "launch") */ Util.spawnCommandLine(command)
		}
	}
}
/* Search Entry Width */
let oldWidth

function newSearchEntryWidth() {
	let searchEntry = Main.overview._searchEntry
	let newWidth = 0.44 * Main.layoutManager.primaryMonitor.width
	
	oldWidth = searchEntry.get_width()
	if (oldWidth < newWidth) searchEntry.set_width(newWidth)
	
	Main.overview._relayout()
}
function oldSearchEntryWidth() {
	Main.overview._searchEntry.set_width(oldWidth)
	
	Main.overview._relayout()
	oldWidth = null
}

/* Extensions API */
let TerminalInst

function init(metadata) {
	MainFile.initTranslations(gettextDomain)
	lockFilePath = GLib.build_filenamev([metadata.path, "managed"])
	
	LAUNCH_TEXT_lc			= LAUNCH_TEXT.toLowerCase()
	LAUNCH_TEXT_lc_i18n		= _(LAUNCH_TEXT).toLowerCase()
	OPEN_TEXT_lc			= OPEN_TEXT.toLowerCase()
	OPEN_TEXT_lc_i18n		= _(OPEN_TEXT).toLowerCase()
	RESTART_TEXT_lc			= RESTART_TEXT.toLowerCase()
	RESTART_TEXT_lc_i18n	= _(RESTART_TEXT).toLowerCase()
	LG_TEXT_lc				= LG_TEXT.toLowerCase()
	LG_TEXT_lc_i18n			= _(LG_TEXT).toLowerCase()
}
function enable() {
	newSearchEntryWidth()
	
	TerminalInst = new TerminalExec()
	Main.overview.addSearchProvider(TerminalInst)
	
	// The extension disables the <Alt>F2 keybinding until the user manually re-set it. If he does, a file is created in the extension's folder that "lock" the behaviour until the extension is removed. The file should be deleted at each manual unactivation of the extension, but that looks like impossible with the actual API.
	if (!GLib.file_test(lockFilePath, GLib.FileTest.EXISTS)) disableShellEntry()
}
function disable() {
	if (!GLib.file_test(lockFilePath, GLib.FileTest.EXISTS)) rEnableShellEntry()
	oldSearchEntryWidth()
	
	Main.overview.removeSearchProvider(TerminalInst)
	TerminalInst = null
}
